import random


class RandomNormalizedArray(object):

    """
    The constructor of RandomNormalizedArray class gets an integer itemCount as an input. Creates a list of
    size itemCount and loops through each element of the list and initializes them with a random number by using random.
    Then, accumulates each element of the list and at the end divides each element with the resulting sum.

    Parameters
    ----------
    itemCount : int
        input defining array size.
     """
    def __init__(self, itemCount: int):
        sum = 0.0
        self.array = []
        for i in range(itemCount):
            self.array.append(random.uniform(0, 1))
            sum += self.array[i]
        for i in range(itemCount):
            self.array[i] /= sum

    """
    Getter for the double list.
   
    Returns
    ----------
    list
        the double list.
     """
    def get(self) -> list:
        return self.array


class Interval(object):
    # A constructor of Interval class which creates a new list.
    def __init__(self):
        self.list = []

    """ The add method adds a new Tuple with given inputs to the list.
    
    Parameters
    ----------
    start : int
        first element of Tuple.
    end : int   
        second element of Tuple.
    """
    def add(self, start: int, end: int):
        self.list.append((start, end))

    """ The getFirst method returns the first element at the list list's given index.
    
    Parameters
    ----------
    index : int 
        to use at getting tuple from list.

    Returns
    ----------
    item
        the first element at the list list's given index.
    """
    def getFirst(self, index: int) -> int:
        return self.list[index][0]

    """ The getLast method returns the last element at the list list's given index.

    Parameters
    ----------
    index : int 
        to use at getting tuple from list.

    Returns
    ----------
    item
        the last element at the list list's given index.
    """
    def getLast(self, index: int) -> int:
        return self.list[index][1]

    """ The size method returns the size of the list list.
    Returns
    ----------
    int
        size of the list list.
    """
    def size(self) -> int:
        return len(self.list)


class Subset(object):

    """
    The constructor of Subset class which takes 3 integer inputs; rangeStart, rangeEnd, and elementCount.
    It initializes variables rangeEnd and elementCount with given inputs, creates 2 arrays; set and multiset with the
    size of given elementCount and multisetCount, which is derived from elementCount, respectively. Then, it assigns
    rangeStart+i to each ith element of set.

    Parameters
    ----------
    rangeStart : int
        input defining start range.
    rangeEnd : int
        input defining ending range.
    elementCount : int
        input element count.
    """
    def __init__(self, rangeStart: int, rangeEnd: int, elementCount: int):
        self.rangeEnd = rangeEnd
        self.elementCount = elementCount
        self.set = []
        self.multiset = []
        for i in range(elementCount):
            self.set.append(rangeStart + i)

    """
    Getter for the set list
    
    Returns
    ----------
    list
        the set list
    """
    def get(self) -> list:
        return self.set

    """
    The getX method takes an integer M as an input. Creates a new list X size of elementCount+2 and assigns 0 to its
    first element. Starting from the second index, it assigns set lists elements to newly
    created list X. Then, assigns M to the last element of X.
    
    Parameters
    ----------
    M
        integer input.
        
    Returns
    ----------
    list
        list of size of elementCount + 2.
    """
    def getX(self, M: int) -> list:
        X = []
        X.append(0)
        for i in range(self.elementCount):
            X.append(self.set[i])
        X.append(M)
        return X

    """
    Getter for the multiset list.

    Returns
    ----------
    list
        the multiset list.
    """
    def getmultiset(self) -> list:
        return self.multiset

    """
    The multiset method takes an integer M as an input. Assigns ith element of set list to even numbered
    indices of multiset list and M - ith element of set list to odd numbered
    indices of multiset list, and i is between 0 and elementCount. Then, assigns M to jth index of multiset
    list. At the end, fill up the rest of the multiset list via different
    indices of set list and sort the multiset list.
    
    Parameters
    ----------
    M
        integer input.
    """
    def multiset(self, M: int):
        for i in range(self.elementCount):
            self.multiset.append(self.set[i])
            self.multiset.append(M - self.set[i])
        self.multiset.append(M)
        for i in range(self.elementCount - 1, 0, -1):
            for k in range(i - 1, -1, -1):
                self.multiset.append(self.set[i] - self.set[k])
        self.multiset.sort()

    """
    The next method generates the next subset.

    Returns
    ----------
    boolean
        true if next subset generation is possible, false otherwise.
    """
    def next(self) -> bool:
        for i in range(self.elementCount - 1, -1, -1):
            self.set[i] = self.set[i] + 1
            if self.set[i] <= self.rangeEnd - self.elementCount + i + 1:
                break
        else:
            return False
        for j in range(i + 1, self.elementCount):
            self.set[j] = self.set[j - 1] + 1
        return True

    """
    The print method prints elements of set list.
    """
    def print(self):
        for i in range(self.elementCount):
            print(self.set[i], end=" ")
        print()



class Swap(object):

    """ The swap method takes a list and two integer numbers i, j. And interchange the given array's
    items at index i and index j.

    Parameters
    ----------
    array : list
        input
    i : int
        first index to swap.
    j : int
        second index to change.
    """

    @staticmethod
    def swap(array: list, i: int, j: int):
        t = array[i]
        array[i] = array[j]
        array[j] = t



from Util.Subset import Subset


class SubsetFromList(Subset):

    """
    A constructor of SubsetFromList class takes an integer list and an integer elementCount as inputs. It initializes
    elementList and elementCount variables with given inputs, then creates 3 arrays; set,indexList, and multiset with the
    size of given elementCount and multisetCount, which is derived from elementCount, respectively. Then, it assigns
    i to each ith element of indexList list and use this index to point at elementList and
    assigns to set list.
    
    Parameters
    ----------
    list : list
        list type input.
    elementCount : int
        input element count.
    """
    def __init__(self, list: list, elementCount: int):
        self.elementList = list
        self.elementCount = elementCount
        self.set = []
        self.indexList = []
        self.multiset = []
        for i in range(elementCount):
            self.indexList.append(i)
            self.set.append(self.elementList[self.indexList[i]])


    """
    The next method generates the next subset from list.

    Returns
    ----------
    boolean
        true if next subset generation from list is possible, false otherwise.
    """
    def next(self) -> bool:
        for i in range(self.elementCount - 1, -1, -1):
            self.indexList[i] = self.indexList[i] + 1
            if self.indexList[i] < len(self.elementList) - self.elementCount + i + 1:
                break
        else:
            return False
        self.set[i] = self.elementList[self.indexList[i]]
        for j in range(i + 1, self.elementCount):
            self.indexList[j] = self.indexList[j - 1] + 1
            self.set[j] = self.elementList[self.indexList[j]]
        return True


from Util.Swap import Swap


class Permutation(object):
    
    """
    A constructor of Permutation class which creates a new list and assigns integer
    numbers starting from 0 to given input n.

    Parameters
    ----------
     n: int
        integer input.
     """
    def __init__(self, n: int):
        self.n = n
        self.a = []
        for i in range(n):
            self.a.append(i)

    """
    The get method returns the list a.

    Returns
    ----------
    list 
        a
    """
    def get(self) -> list:
        return self.a

    """
     The next method generates next permutation for the list a.

    Returns
    ----------
    boolean
        true if next permutation is possible, false otherwise.
     """
    def next(self) -> bool:
        i = self.n - 2
        while i >= 0 and self.a[i] >= self.a[i + 1]:
            i = i - 1
        if i == -1:
            return False
        j = self.n - 1
        while self.a[i] >= self.a[j]:
            j = j - 1
        Swap.swap(self.a, i, j)
        k = i + 1
        j = self.n - 1
        while k < j:
            Swap.swap(self.a, j, k)
            k = k + 1
            j = j - 1
        return True